@@ -1,8 +1,8 @@
-## is.ultrametric.R (2006-10-04)
+## is.ultrametric.R (2007-12-18)
 
 ##   Test if a Tree is Ultrametric
 
-## Copyright 2003-2006 Emmanuel Paradis
+## Copyright 2003-2007 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -24,9 +24,7 @@ is.ultrametric <- function(phy, tol = .Machine$double.eps^0.5)
     for (i in 1:dim(phy$edge)[1])
       xx[phy$edge[i, 2]] <- xx[phy$edge[i, 1]] + phy$edge.length[i]
 
-    if (isTRUE(all.equal(var(xx[1:n]), 0, tol = tol))) return(TRUE)
-
-    if (identical(all.equal(var(xx[1:n]), 0,
-                            tolerance = tol), TRUE)) return(TRUE)
-    else return(FALSE)
+    if (identical(all.equal.numeric(var(xx[1:n]),
+                                    0, tolerance = tol), TRUE)) TRUE
+    else FALSE
 }
